unit RichText;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  CustSaveIt, AlmostSave, MDISaveIt, StdCtrls, ComCtrls, Buttons,
  ExtCtrls, GenericSave;

type
  TRichChild = class(TForm)
    RichChildSaveIt: TMDIChildSaveIt;
    RichEdit1: TRichEdit;
    SpeedBar: TPanel;
    Bevel1: TBevel;
    BoldButton: TSpeedButton;
    ItalicButton: TSpeedButton;
    UnderlineButton: TSpeedButton;
    BulletsButton: TSpeedButton;
    FontName: TComboBox;
    FontSize: TEdit;
    UpDown1: TUpDown;
    procedure FontNameChange(Sender: TObject);
    procedure FontSizeChange(Sender: TObject);
    procedure BoldButtonClick(Sender: TObject);
    procedure ItalicButtonClick(Sender: TObject);
    procedure UnderlineButtonClick(Sender: TObject);
    procedure BulletsButtonClick(Sender: TObject);
    procedure RichEdit1SelectionChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RichChildSaveItDataCheck(Sender: TObject;
      var Has_Data: Boolean);
    procedure RichChildSaveItDataSave(Sender: TObject; FileName: String;
      var Success: Boolean);
    procedure RichChildSaveItDataLoad(Sender: TObject; FileName: String;
      var Success: Boolean);
    procedure RichEdit1Change(Sender: TObject);
  private
    FUpdating: Boolean;

    function CurrText: TTextAttributes;
    procedure GetFontNames;
  end;

var
  RichChild: TRichChild;

implementation

{$R *.DFM}

// *** Begin Ignorable Code ***

function TRichChild.CurrText: TTextAttributes;
begin
  if RichEdit1.SelLength > 0 then
    Result := RichEdit1.SelAttributes
  else
    Result := RichEdit1.DefAttributes;
end;

function EnumFontsProc(var LogFont: TLogFont; var TextMetric: TTextMetric;
  FontType: Integer; Data: Pointer): Integer; stdcall;
begin
  TStrings(Data).Add(LogFont.lfFaceName);
  Result := 1;
end;

procedure TRichChild.GetFontNames;
var
  DC: HDC;
begin
  DC := GetDC(0);
  EnumFonts(DC, nil, @EnumFontsProc, Pointer(FontName.Items));
  ReleaseDC(0, DC);
  FontName.Sorted := True;
end;

// *** IDE Maintained Methods ***

procedure TRichChild.FontNameChange(Sender: TObject);
begin
  if FUpdating then Exit;
  CurrText.Name := FontName.Items[FontName.ItemIndex];
end;

procedure TRichChild.FontSizeChange(Sender: TObject);
begin
  if FUpdating then Exit;
  CurrText.Size := StrToInt(FontSize.Text);
end;

procedure TRichChild.BoldButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  if BoldButton.Down then
    CurrText.Style := CurrText.Style + [fsBold]
  else
    CurrText.Style := CurrText.Style - [fsBold];
end;

procedure TRichChild.ItalicButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  if ItalicButton.Down then
    CurrText.Style := CurrText.Style + [fsItalic]
  else
    CurrText.Style := CurrText.Style - [fsItalic];
end;

procedure TRichChild.UnderlineButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  if UnderlineButton.Down then
    CurrText.Style := CurrText.Style + [fsUnderline]
  else
    CurrText.Style := CurrText.Style - [fsUnderline];
end;

procedure TRichChild.BulletsButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  RichEdit1.Paragraph.Numbering := TNumberingStyle(BulletsButton.Down);
end;

procedure TRichChild.RichEdit1SelectionChange(Sender: TObject);
begin
  with RichEdit1.Paragraph do
  try
    FUpdating := True;
    BoldButton.Down := fsBold in RichEdit1.SelAttributes.Style;
    ItalicButton.Down := fsItalic in RichEdit1.SelAttributes.Style;
    UnderlineButton.Down := fsUnderline in RichEdit1.SelAttributes.Style;
    BulletsButton.Down := Boolean(Numbering);
    FontSize.Text := IntToStr(RichEdit1.SelAttributes.Size);
    FontName.Text := RichEdit1.SelAttributes.Name;
  finally
    FUpdating := False;
  end;
end;

procedure TRichChild.FormCreate(Sender: TObject);
begin
  GetFontNames;
end;

// *** End Ignorable Code ***

// *** Begin Important Code ***

procedure TRichChild.RichChildSaveItDataCheck(Sender: TObject;
  var Has_Data: Boolean);
begin
  Has_Data := Length(Trim(RichEdit1.Text)) > 0;
end;

procedure TRichChild.RichChildSaveItDataSave(Sender: TObject;
  FileName: String; var Success: Boolean);
begin
  // Save To File
  RichEdit1.Lines.SaveToFile(FileName);
end;

procedure TRichChild.RichChildSaveItDataLoad(Sender: TObject;
  FileName: String; var Success: Boolean);
begin
  // Load From File
  RichEdit1.Lines.LoadFromFile(FileName);
end;

procedure TRichChild.RichEdit1Change(Sender: TObject);
begin
  RichChildSaveIt.Modified;
end;

// *** End Important Code ***

end.
